@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
-from libnmap.objects.report import NmapReport
 from libnmap.objects.host import NmapHost
+from libnmap.objects.report import NmapReport
 from libnmap.objects.service import NmapService
 
-__all__ = ['NmapReport', 'NmapHost', 'NmapService']
+__all__ = ["NmapReport", "NmapHost", "NmapService"]
