@@ -2,21 +2,20 @@
 # -*- coding: utf-8 -*-
 
 import os
+import platform
 import shlex
 import subprocess
+import warnings
 from threading import Thread
 from xml.dom import pulldom
-import warnings
-import platform
+
 try:
     import pwd
 except ImportError:
     pass
 
 
-__all__ = [
-    'NmapProcess'
-]
+__all__ = ["NmapProcess"]
 
 
 class NmapTask(object):
@@ -32,13 +31,13 @@ class NmapTask(object):
     is built during scan execution
     """
 
-    def __init__(self, name, starttime=0, extrainfo=''):
+    def __init__(self, name, starttime=0, extrainfo=""):
         self.name = name
         self.etc = 0
         self.progress = 0
         self.percent = 0
         self.remaining = 0
-        self.status = 'started'
+        self.status = "started"
         self.starttime = starttime
         self.endtime = 0
         self.extrainfo = extrainfo
@@ -56,8 +55,14 @@ class NmapProcess(Thread):
     parsed out via the NmapParser class from libnmap.parser module.
     """
 
-    def __init__(self, targets="127.0.0.1",
-                 options="-sT", event_callback=None, safe_mode=True, fqp=None):
+    def __init__(
+        self,
+        targets="127.0.0.1",
+        options="-sT",
+        event_callback=None,
+        safe_mode=True,
+        fqp=None,
+    ):
         """
         Constructor of NmapProcess class.
 
@@ -85,11 +90,22 @@ class NmapProcess(Thread):
 
         """
         Thread.__init__(self)
-        unsafe_opts = set(['-oG', '-oN', '-iL', '-oA', '-oS', '-oX',
-                           '--iflist', '--resume', '--stylesheet',
-                           '--datadir'])
+        unsafe_opts = set(
+            [
+                "-oG",
+                "-oN",
+                "-iL",
+                "-oA",
+                "-oS",
+                "-oX",
+                "--iflist",
+                "--resume",
+                "--stylesheet",
+                "--datadir",
+            ]
+        )
         # more reliable than just using os.name() (cygwin)
-        self.__is_windows = platform.system() == 'Windows'
+        self.__is_windows = platform.system() == "Windows"
         if fqp:
             if os.path.isfile(fqp) and os.access(fqp, os.X_OK):
                 self.__nmap_binary = fqp
@@ -101,31 +117,38 @@ class NmapProcess(Thread):
         self.__nmap_fixed_options = "-oX - -vvv --stats-every 1s"
 
         if self.__nmap_binary is None:
-            raise EnvironmentError(1, "nmap is not installed or could "
-                                      "not be found in system path")
+            em = "nmap is not installed or could not be found in system path"
+            raise EnvironmentError(1, em)
 
         if isinstance(targets, str):
-            self.__nmap_targets = targets.replace(" ", "").split(',')
+            self.__nmap_targets = targets.replace(" ", "").split(",")
         elif isinstance(targets, list):
             self.__nmap_targets = targets
         else:
-            raise Exception("Supplied target list should be either a "
-                            "string or a list")
+            raise Exception(
+                "Supplied target list should be either a string or a list"
+            )
 
         self._nmap_options = set(options.split())
         if safe_mode and not self._nmap_options.isdisjoint(unsafe_opts):
-            raise Exception("unsafe options activated while safe_mode "
-                            "is set True")
+            raise Exception(
+                "unsafe options activated while safe_mode is set True"
+            )
         self.__nmap_dynamic_options = options
-        self.__sudo_run = ''
+        self.__sudo_run = ""
         self.__nmap_command_line = self.get_command_line()
 
         if event_callback and callable(event_callback):
             self.__nmap_event_callback = event_callback
         else:
             self.__nmap_event_callback = None
-        (self.DONE, self.READY, self.RUNNING,
-         self.CANCELLED, self.FAILED) = range(5)
+        (
+            self.DONE,
+            self.READY,
+            self.RUNNING,
+            self.CANCELLED,
+            self.FAILED,
+        ) = range(5)
         self._run_init()
 
     def _run_init(self):
@@ -136,12 +159,12 @@ class NmapProcess(Thread):
         self.__state = self.RUNNING
         self.__starttime = 0
         self.__endtime = 0
-        self.__version = ''
-        self.__elapsed = ''
-        self.__summary = ''
-        self.__stdout = ''
-        self.__stderr = ''
-        self.__current_task = ''
+        self.__version = ""
+        self.__elapsed = ""
+        self.__summary = ""
+        self.__stdout = ""
+        self.__stderr = ""
+        self.__current_task = ""
         self.__nmap_tasks = {}
 
     def _whereis(self, program):
@@ -156,11 +179,11 @@ class NmapProcess(Thread):
 
         :todo: add a default path list in case PATH is empty.
         """
-        split_char = ';' if self.__is_windows else ':'
-        program = program + '.exe' if self.__is_windows else program
-        for path in os.environ.get('PATH', '').split(split_char):
-            if (os.path.exists(os.path.join(path, program)) and not
-                    os.path.isdir(os.path.join(path, program))):
+        split_char = ";" if self.__is_windows else ":"
+        program = program + ".exe" if self.__is_windows else program
+        for path in os.environ.get("PATH", "").split(split_char):
+            _file_path = os.path.join(path, program)
+            if os.path.exists(_file_path) and not os.path.isdir(_file_path):
                 return os.path.join(path, program)
         return None
 
@@ -171,13 +194,25 @@ class NmapProcess(Thread):
         :return: the full nmap command line to run
         :rtype: string
         """
-        return ("{0} {1} {2} {3} {4}".format(self.__sudo_run,
-                                             self.__nmap_binary,
-                                             self.__nmap_fixed_options,
-                                             self.__nmap_dynamic_options,
-                                             " ".join(self.__nmap_targets)))
+        return "{0} {1} {2} {3} {4}".format(
+            self.__sudo_run,
+            self.__nmap_binary,
+            self.__nmap_fixed_options,
+            self.__nmap_dynamic_options,
+            " ".join(self.__nmap_targets),
+        )
 
-    def sudo_run(self, run_as='root'):
+    def _ensure_user_exists(self, username=""):
+        try:
+            pwd.getpwnam(username).pw_uid
+        except KeyError as eobj:
+            _exmsg = (
+                "Username {0} does not exists. Please supply"
+                " a valid username: {1}".format(username, eobj)
+            )
+            raise EnvironmentError(_exmsg)
+
+    def sudo_run(self, run_as="root"):
         """
         Public method enabling the library's user to run the scan with
         priviledges via sudo. The sudo configuration should be set manually
@@ -190,18 +225,16 @@ class NmapProcess(Thread):
         :return: return code from nmap execution
         """
         sudo_user = run_as.split().pop()
-        try:
-            pwd.getpwnam(sudo_user).pw_uid
-        except KeyError:
-            _exmsg = ("Username {0} does not exists. Please supply"
-                      " a valid username".format(run_as))
-            raise EnvironmentError(_exmsg)
+        self._ensure_user_exists(sudo_user)
 
         sudo_path = self._whereis("sudo")
         if sudo_path is None:
-            raise EnvironmentError(2, "sudo is not installed or "
-                                      "could not be found in system path: "
-                                      "cannot run nmap with sudo")
+            raise EnvironmentError(
+                2,
+                "sudo is not installed or "
+                "could not be found in system path: "
+                "cannot run nmap with sudo",
+            )
 
         self.__sudo_run = "{0} -u {1}".format(sudo_path, sudo_user)
         rc = self.run()
@@ -209,7 +242,7 @@ class NmapProcess(Thread):
 
         return rc
 
-    def sudo_run_background(self, run_as='root'):
+    def sudo_run_background(self, run_as="root"):
         """
         Public method enabling the library's user to run in background a
         nmap scan with priviledges via sudo.
@@ -223,18 +256,16 @@ class NmapProcess(Thread):
         :return: return code from nmap execution
         """
         sudo_user = run_as.split().pop()
-        try:
-            pwd.getpwnam(sudo_user).pw_uid
-        except KeyError:
-            _exmsg = ("Username {0} does not exists. Please supply"
-                      " a valid username".format(run_as))
-            raise EnvironmentError(_exmsg)
+        self._ensure_user_exists(sudo_user)
 
         sudo_path = self._whereis("sudo")
         if sudo_path is None:
-            raise EnvironmentError(2, "sudo is not installed or "
-                                      "could not be found in system path: "
-                                      "cannot run nmap with sudo")
+            raise EnvironmentError(
+                2,
+                "sudo is not installed or "
+                "could not be found in system path: "
+                "cannot run nmap with sudo",
+            )
 
         self.__sudo_run = "{0} -u {1}".format(sudo_path, sudo_user)
         super(NmapProcess, self).start()
@@ -253,27 +284,31 @@ class NmapProcess(Thread):
         return: return code from nmap execution
         """
         self._run_init()
-        _tmp_cmdline = self.__build_windows_cmdline() if self.__is_windows \
+        _tmp_cmdline = (
+            self.__build_windows_cmdline()
+            if self.__is_windows
             else shlex.split(self.__nmap_command_line)
+        )
         try:
-            self.__nmap_proc = subprocess.Popen(args=_tmp_cmdline,
-                                                stdout=subprocess.PIPE,
-                                                stderr=subprocess.PIPE,
-                                                universal_newlines=True,
-                                                bufsize=0)
+            self.__nmap_proc = subprocess.Popen(
+                args=_tmp_cmdline,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+                universal_newlines=True,
+                bufsize=0,
+            )
             self.__state = self.RUNNING
-        except OSError:
+        except OSError as emsg:
             self.__state = self.FAILED
-            raise EnvironmentError(1, "nmap is not installed or could "
-                                      "not be found in system path")
+            raise EnvironmentError(
+                1,
+                "nmap is not installed or could "
+                "not be found in system path: {0}".format(emsg),
+            )
 
         while self.__nmap_proc.poll() is None:
-            for streamline in iter(self.__nmap_proc.stdout.readline, ''):
-                self.__stdout += streamline
-                evnt = self.__process_event(streamline)
-                if self.__nmap_event_callback and evnt:
-                    self.__nmap_event_callback(self)
-
+            self.__process_nmap_proc_stdout()
+        self.__process_nmap_proc_stdout()
         self.__stderr += self.__nmap_proc.stderr.read()
 
         self.__nmap_rc = self.__nmap_proc.poll()
@@ -312,8 +347,11 @@ class NmapProcess(Thread):
 
         :return: True if nmap process is not running anymore.
         """
-        return (self.state == self.DONE or self.state == self.FAILED or
-                self.state == self.CANCELLED)
+        return (
+            self.state == self.DONE
+            or self.state == self.FAILED
+            or self.state == self.CANCELLED
+        )
 
     def has_failed(self):
         """
@@ -340,6 +378,13 @@ class NmapProcess(Thread):
         if self.__nmap_proc.poll() is None:
             self.__nmap_proc.kill()
 
+    def __process_nmap_proc_stdout(self):
+        for streamline in iter(self.__nmap_proc.stdout.readline, ""):
+            self.__stdout += streamline
+            evnt = self.__process_event(streamline)
+            if self.__nmap_event_callback and evnt:
+                self.__nmap_event_callback(self)
+
     def __process_event(self, eventdata):
         """
         Private method called while nmap process is running. It enables the
@@ -362,55 +407,65 @@ class NmapProcess(Thread):
             edomdoc = pulldom.parseString(eventdata)
             for xlmnt, xmlnode in edomdoc:
                 if xlmnt is not None and xlmnt == pulldom.START_ELEMENT:
-                    if (xmlnode.nodeName == 'taskbegin' and
-                            xmlnode.attributes.keys()):
+                    if (
+                        xmlnode.nodeName == "taskbegin"
+                        and xmlnode.attributes.keys()
+                    ):
                         xt = xmlnode.attributes
-                        taskname = xt['task'].value
-                        starttime = xt['time'].value
-                        xinfo = ''
-                        if 'extrainfo' in xt.keys():
-                            xinfo = xt['extrainfo'].value
+                        taskname = xt["task"].value
+                        starttime = xt["time"].value
+                        xinfo = ""
+                        if "extrainfo" in xt.keys():
+                            xinfo = xt["extrainfo"].value
                         newtask = NmapTask(taskname, starttime, xinfo)
                         self.__nmap_tasks[newtask.name] = newtask
                         self.__current_task = newtask.name
                         rval = True
-                    elif (xmlnode.nodeName == 'taskend' and
-                            xmlnode.attributes.keys()):
+                    elif (
+                        xmlnode.nodeName == "taskend"
+                        and xmlnode.attributes.keys()
+                    ):
                         xt = xmlnode.attributes
-                        tname = xt['task'].value
-                        xinfo = ''
-                        self.__nmap_tasks[tname].endtime = xt['time'].value
-                        if 'extrainfo' in xt.keys():
-                            xinfo = xt['extrainfo'].value
+                        tname = xt["task"].value
+                        xinfo = ""
+                        self.__nmap_tasks[tname].endtime = xt["time"].value
+                        if "extrainfo" in xt.keys():
+                            xinfo = xt["extrainfo"].value
                         self.__nmap_tasks[tname].extrainfo = xinfo
                         self.__nmap_tasks[tname].status = "ended"
                         rval = True
-                    elif (xmlnode.nodeName == 'taskprogress' and
-                            xmlnode.attributes.keys()):
+                    elif (
+                        xmlnode.nodeName == "taskprogress"
+                        and xmlnode.attributes.keys()
+                    ):
                         xt = xmlnode.attributes
-                        tname = xt['task'].value
-                        percent = xt['percent'].value
-                        etc = xt['etc'].value
-                        remaining = xt['remaining'].value
-                        updated = xt['time'].value
+                        tname = xt["task"].value
+                        percent = xt["percent"].value
+                        etc = xt["etc"].value
+                        remaining = xt["remaining"].value
+                        updated = xt["time"].value
                         self.__nmap_tasks[tname].percent = percent
                         self.__nmap_tasks[tname].progress = percent
                         self.__nmap_tasks[tname].etc = etc
                         self.__nmap_tasks[tname].remaining = remaining
                         self.__nmap_tasks[tname].updated = updated
                         rval = True
-                    elif (xmlnode.nodeName == 'nmaprun' and
-                            xmlnode.attributes.keys()):
-                        self.__starttime = xmlnode.attributes['start'].value
-                        self.__version = xmlnode.attributes['version'].value
+                    elif (
+                        xmlnode.nodeName == "nmaprun"
+                        and xmlnode.attributes.keys()
+                    ):
+                        self.__starttime = xmlnode.attributes["start"].value
+                        self.__version = xmlnode.attributes["version"].value
                         rval = True
-                    elif (xmlnode.nodeName == 'finished' and
-                            xmlnode.attributes.keys()):
-                        self.__endtime = xmlnode.attributes['time'].value
-                        self.__elapsed = xmlnode.attributes['elapsed'].value
-                        self.__summary = xmlnode.attributes['summary'].value
+                    elif (
+                        xmlnode.nodeName == "finished"
+                        and xmlnode.attributes.keys()
+                    ):
+                        self.__endtime = xmlnode.attributes["time"].value
+                        self.__elapsed = xmlnode.attributes["elapsed"].value
+                        self.__summary = xmlnode.attributes["summary"].value
                         rval = True
-        except:
+        except Exception:
             pass
         return rval
 
@@ -433,7 +488,7 @@ class NmapProcess(Thread):
 
         :return: string
         """
-        return self.__nmap_command_line or ''
+        return self.__nmap_command_line or ""
 
     @property
     def targets(self):
@@ -484,8 +539,11 @@ class NmapProcess(Thread):
 
         :return: string. Unix timestamp
         """
-        warnings.warn("data collected from finished events are deprecated."
-                      "Use NmapParser.parse()", DeprecationWarning)
+        warnings.warn(
+            "data collected from finished events are deprecated."
+            "Use NmapParser.parse()",
+            DeprecationWarning,
+        )
         return self.__endtime
 
     @property
@@ -495,8 +553,11 @@ class NmapProcess(Thread):
 
         :return: string
         """
-        warnings.warn("data collected from finished events are deprecated."
-                      "Use NmapParser.parse()", DeprecationWarning)
+        warnings.warn(
+            "data collected from finished events are deprecated."
+            "Use NmapParser.parse()",
+            DeprecationWarning,
+        )
         return self.__elapsed
 
     @property
@@ -506,8 +567,11 @@ class NmapProcess(Thread):
 
         :return: string
         """
-        warnings.warn("data collected from finished events are deprecated."
-                      "Use NmapParser.parse()", DeprecationWarning)
+        warnings.warn(
+            "data collected from finished events are deprecated."
+            "Use NmapParser.parse()",
+            DeprecationWarning,
+        )
         return self.__summary
 
     @property
@@ -599,24 +663,30 @@ def main():
     def mycallback(nmapscan=None):
         if nmapscan.is_running() and nmapscan.current_task:
             ntask = nmapscan.current_task
-            print("Task {0} ({1}): ETC: {2} DONE: {3}%".format(ntask.name,
-                                                               ntask.status,
-                                                               ntask.etc,
-                                                               ntask.progress))
-    nm = NmapProcess("scanme.nmap.org",
-                     options="-A",
-                     event_callback=mycallback)
+            print(
+                "Task {0} ({1}): ETC: {2} DONE: {3}%".format(
+                    ntask.name, ntask.status, ntask.etc, ntask.progress
+                )
+            )
+
+    nm = NmapProcess(
+        "scanme.nmap.org", options="-A", event_callback=mycallback
+    )
     rc = nm.run()
     if rc == 0:
-        print("Scan started at {0} nmap version: {1}").format(nm.starttime,
-                                                              nm.version)
-        print("state: {0} (rc: {1})").format(nm.state, nm.rc)
-        print("results size: {0}").format(len(nm.stdout))
-        print("Scan ended {0}: {1}").format(nm.endtime, nm.summary)
+        print(
+            "Scan started at {0} nmap version: {1}".format(
+                nm.starttime, nm.version
+            )
+        )
+        print("state: {0} (rc: {1})".format(nm.state, nm.rc))
+        print("results size: {0}".format(len(nm.stdout)))
+        print("Scan ended {0}: {1}".format(nm.endtime, nm.summary))
     else:
-        print("state: {0} (rc: {1})").format(nm.state, nm.rc)
-        print("Error: {stderr}").format(stderr=nm.stderr)
-        print("Result: {0}").format(nm.stdout)
+        print("state: {0} (rc: {1})".format(nm.state, nm.rc))
+        print("Error: {stderr}".format(stderr=nm.stderr))
+        print("Result: {0}".format(nm.stdout))
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     main()
